@@ -1,6 +1,12 @@
 import os
 
-from sqlmesh.core.config import AirflowSchedulerConfig, Config, DuckDBConnectionConfig
+from sqlmesh.core.config import (
+    AirflowSchedulerConfig,
+    AutoCategorizationMode,
+    CategorizerConfig,
+    Config,
+    DuckDBConnectionConfig,
+)
 
 DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
 
@@ -8,6 +14,13 @@ DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
 # An in memory DuckDB config.
 config = Config(connections=DuckDBConnectionConfig())
 
+
+# A configuration used for SQLMesh tests.
+test_config = Config(
+    connections=DuckDBConnectionConfig(),
+    auto_categorize_changes=CategorizerConfig(sql=AutoCategorizationMode.SEMI),
+)
+
 # A stateful DuckDB config.
 local_config = Config(
     connections=DuckDBConnectionConfig(database=f"{DATA_DIR}/local.duckdb"),
