@@ -6,7 +6,7 @@
 
 SELECT
   waiter_id::INT AS waiter_id,
-  revenue::DOUBLE AS revenue
+  revenue::DOUBLE AS {{ var("top_waiters:revenue") }}
 FROM {{ ref('waiter_revenue_by_day') }}
 WHERE
   ds = (
@@ -16,4 +16,4 @@ WHERE
   )
 ORDER BY
   revenue DESC
-LIMIT {{ var('top_waiters_limit') }}
+LIMIT {{ var('top_waiters:limit') }}
