@@ -7,7 +7,7 @@ from sqlmesh.core.engine_adapter.base import (
 )
 from sqlmesh.core.engine_adapter.bigquery import BigQueryEngineAdapter
 from sqlmesh.core.engine_adapter.databricks import DatabricksSparkSessionEngineAdapter
-from sqlmesh.core.engine_adapter.databricks_api import DatabricksAPIEngineAdapter
+from sqlmesh.core.engine_adapter.databricks_api import DatabricksSQLEngineAdapter
 from sqlmesh.core.engine_adapter.duckdb import DuckDBEngineAdapter
 from sqlmesh.core.engine_adapter.redshift import RedshiftEngineAdapter
 from sqlmesh.core.engine_adapter.shared import TransactionType
@@ -33,7 +33,6 @@ def create_engine_adapter(
     dialect = dialect.lower()
     if dialect == "postgresql":
         dialect = "postgres"
-    # TODO: Update in follow up PR
     if dialect == "databricks":
         try:
             from pyspark.sql import SparkSession
@@ -44,9 +43,9 @@ def create_engine_adapter(
                     t.Type[EngineAdapter]
                 ] = DatabricksSparkSessionEngineAdapter
             else:
-                engine_adapter = DatabricksAPIEngineAdapter
+                engine_adapter = DatabricksSQLEngineAdapter
         except ImportError:
-            engine_adapter = DatabricksAPIEngineAdapter
+            engine_adapter = DatabricksSQLEngineAdapter
     else:
         engine_adapter = DIALECT_TO_ENGINE_ADAPTER.get(dialect)
     if engine_adapter is None:
