@@ -5,12 +5,14 @@ from enum import Enum
 
 from croniter import croniter
 from pydantic import Field, root_validator, validator
-from sqlglot import exp, maybe_parse, parse_one
+from sqlglot import exp, maybe_parse
 
+import sqlmesh.core.dialect as d
 from sqlmesh.core.model.kind import (
     IncrementalByTimeRangeKind,
     IncrementalByUniqueKeyKind,
     ModelKind,
+    ModelKindName,
     TimeColumn,
     model_kind_validator,
 )
@@ -33,7 +35,7 @@ class IntervalUnit(str, Enum):
     MINUTE = "minute"
 
 
-HookCall = t.Tuple[str, t.Dict[str, exp.Expression]]
+HookCall = t.Union[exp.Expression, t.Tuple[str, t.Dict[str, exp.Expression]]]
 AuditReference = t.Tuple[str, t.Dict[str, exp.Expression]]
 
 
@@ -41,7 +43,7 @@ class ModelMeta(PydanticModel):
     """Metadata for models which can be defined in SQL."""
 
     name: str
-    kind: ModelKind = IncrementalByTimeRangeKind()
+    kind: ModelKind = ModelKind(name=ModelKindName.VIEW)
     dialect: str = ""
     cron: str = "@daily"
     owner: t.Optional[str]
@@ -62,8 +64,8 @@ class ModelMeta(PydanticModel):
 
     _model_kind_validator = model_kind_validator
 
-    @validator("pre", "post", "audits", pre=True)
-    def _value_or_tuple_with_args_validator(cls, v: t.Any) -> t.Any:
+    @validator("audits", pre=True)
+    def _audits_validator(cls, v: t.Any) -> t.Any:
         def extract(v: exp.Expression) -> t.Tuple[str, t.Dict[str, str]]:
             kwargs = {}
 
@@ -74,7 +76,7 @@ class ModelMeta(PydanticModel):
                 func = v.sql_name()
                 args = list(v.args.values())
             else:
-                return v.name, {}
+                return v.name.lower(), {}
 
             for arg in args:
                 if not isinstance(arg, exp.EQ):
@@ -82,7 +84,7 @@ class ModelMeta(PydanticModel):
                         f"Function '{func}' must be called with key-value arguments like {func}(arg=value)."
                     )
                 kwargs[arg.left.name] = arg.right
-            return (func, kwargs)
+            return (func.lower(), kwargs)
 
         if isinstance(v, (exp.Tuple, exp.Array)):
             return [extract(i) for i in v.expressions]
@@ -93,9 +95,9 @@ class ModelMeta(PydanticModel):
         if isinstance(v, list):
             return [
                 (
-                    entry[0],
+                    entry[0].lower(),
                     {
-                        key: parse_one(value) if isinstance(value, str) else value
+                        key: d.parse(value)[0] if isinstance(value, str) else value
                         for key, value in entry[1].items()
                     },
                 )
@@ -103,6 +105,65 @@ class ModelMeta(PydanticModel):
             ]
         return v
 
+    @validator("pre", "post", pre=True)
+    def _value_or_tuple_with_args_validator(cls, v: t.Any) -> t.Any:
+        def extract(v: exp.Expression) -> t.Union[exp.Expression, t.Tuple[str, t.Dict[str, str]]]:
+            kwargs = {}
+
+            if isinstance(v, exp.Anonymous):
+                func = v.name
+                args = v.expressions
+            elif not isinstance(v, d.Jinja) and isinstance(v, exp.Func):
+                func = v.sql_name()
+                args = list(v.args.values())
+            elif isinstance(v, exp.Column):
+                return v.name.lower(), {}
+            else:
+                return v
+
+            for arg in args:
+                if not isinstance(arg, exp.EQ):
+                    raise ConfigError(
+                        f"Function '{func}' must be called with key-value arguments like {func}(arg=value)."
+                    )
+                kwargs[arg.left.name] = arg.right
+            return (func.lower(), kwargs)
+
+        if isinstance(v, (exp.Tuple, exp.Array)):
+            items: t.List[t.Any] = []
+            for item in v.expressions:
+                if isinstance(item, exp.Literal):
+                    items.append(d.parse(item.this)[0])
+                elif isinstance(item, exp.Column) and isinstance(item.this, exp.Identifier):
+                    items.append(d.parse(item.this.this)[0])
+                else:
+                    items.append(extract(item))
+            return items
+        if isinstance(v, exp.Paren):
+            return [extract(v.this)]
+        if isinstance(v, exp.Expression):
+            return [extract(v)]
+        if isinstance(v, list) and v:
+            transformed = []
+            for entry in v:
+                if isinstance(entry, exp.Expression):
+                    transformed.append(extract(entry))
+                elif isinstance(entry, str):
+                    transformed.append(d.parse(entry)[0])
+                else:
+                    transformed.append(
+                        (
+                            entry[0].lower(),
+                            {
+                                key: d.parse(value)[0] if isinstance(value, str) else value
+                                for key, value in entry[1].items()
+                            },
+                        )
+                    )
+            return transformed
+
+        return v
+
     @validator("partitioned_by_", pre=True)
     def _value_or_tuple_validator(cls, v: t.Any) -> t.Any:
         if isinstance(v, (exp.Tuple, exp.Array)):
