@@ -1,10 +1,11 @@
 from __future__ import annotations
 
 import typing as t
-from collections import defaultdict, deque
+from collections import defaultdict
 from enum import Enum
 
 from sqlmesh.core import scheduler
+from sqlmesh.core.config import CategorizerConfig
 from sqlmesh.core.context_diff import ContextDiff
 from sqlmesh.core.environment import Environment
 from sqlmesh.core.snapshot import (
@@ -12,6 +13,7 @@ from sqlmesh.core.snapshot import (
     Snapshot,
     SnapshotChangeCategory,
     SnapshotId,
+    categorize_change,
     merge_intervals,
 )
 from sqlmesh.core.state_sync import StateReader
@@ -52,6 +54,8 @@ class Plan:
         is_dev: Whether this plan is for development purposes.
         forward_only: Whether the purpose of the plan is to make forward only changes.
         environment_ttl: The period of time that a development environment should exist before being deleted.
+        categorizer_config: Auto categorization settings.
+        auto_categorization_enabled: Whether to apply auto categorization.
     """
 
     def __init__(
@@ -68,6 +72,8 @@ class Plan:
         is_dev: bool = False,
         forward_only: bool = False,
         environment_ttl: t.Optional[str] = None,
+        categorizer_config: t.Optional[CategorizerConfig] = None,
+        auto_categorization_enabled: bool = True,
     ):
         self.context_diff = context_diff
         self.override_start = start is not None
@@ -78,12 +84,14 @@ class Plan:
         self.is_dev = is_dev
         self.forward_only = forward_only
         self.environment_ttl = environment_ttl
+        self.categorizer_config = categorizer_config or CategorizerConfig()
+        self.auto_categorization_enabled = auto_categorization_enabled
         self._start = start if start or not (is_dev and forward_only) else yesterday_ds()
         self._end = end if end or not is_dev else now()
         self._apply = apply
         self._dag = dag
         self._state_reader = state_reader
-        self._missing_intervals: t.Optional[t.Dict[str, Intervals]] = None
+        self.__missing_intervals: t.Optional[t.Dict[str, Intervals]] = None
         self._restatements: t.Set[str] = set()
 
         if restate_models and context_diff.new_snapshots:
@@ -101,14 +109,15 @@ class Plan:
 
         self._add_restatements(restate_models or [])
 
-        self._ensure_valid_end(self._end)
+        self._ensure_valid_date_range(self._start, self._end)
         self._ensure_no_forward_only_revert()
         self._ensure_no_forward_only_new_models()
-        self._ensure_no_forward_only_seed_models()
 
-        categorized_snapshots = self._categorize_snapshots()
-        self.added_and_directly_modified = categorized_snapshots[0]
-        self.indirectly_modified = categorized_snapshots[1]
+        directly_indirectly_modified = self._build_directly_and_indirectly_modified()
+        self.directly_modified = directly_indirectly_modified[0]
+        self.indirectly_modified = directly_indirectly_modified[1]
+
+        self._categorize_snapshots()
 
         self._categorized: t.Optional[t.List[Snapshot]] = None
         self._uncategorized: t.Optional[t.List[Snapshot]] = None
@@ -117,25 +126,37 @@ class Plan:
     def categorized(self) -> t.List[Snapshot]:
         """Returns the already categorized snapshots."""
         if self._categorized is None:
-            self._categorized = [s for s in self.added_and_directly_modified if s.version]
+            self._categorized = [s for s in self.directly_modified if s.version]
         return self._categorized
 
     @property
     def uncategorized(self) -> t.List[Snapshot]:
         """Returns the uncategorized snapshots."""
         if self._uncategorized is None:
-            self._uncategorized = [s for s in self.added_and_directly_modified if not s.version]
+            self._uncategorized = [s for s in self.directly_modified if not s.version]
         return self._uncategorized
 
     @property
     def start(self) -> TimeLike:
         """Returns the start of the plan or the earliest date of all snapshots."""
-        return self._start or scheduler.earliest_start_date(self.snapshots)
+        return self._start or (
+            min(
+                start
+                for intervals_per_model in self._missing_intervals.values()
+                for start, _ in intervals_per_model
+            )
+            if self._missing_intervals
+            else yesterday_ds()
+        )
 
     @start.setter
     def start(self, new_start: TimeLike) -> None:
+        self._ensure_valid_date_range(new_start, self._end)
+        self.set_start(new_start)
+
+    def set_start(self, new_start: TimeLike) -> None:
         self._start = new_start
-        self._missing_intervals = None
+        self.__missing_intervals = None
 
     @property
     def end(self) -> TimeLike:
@@ -144,13 +165,13 @@ class Plan:
 
     @end.setter
     def end(self, new_end: TimeLike) -> None:
-        self._ensure_valid_end(new_end)
+        self._ensure_valid_date_range(self._start, new_end)
         self._end = new_end
-        self._missing_intervals = None
+        self.__missing_intervals = None
 
     @property
-    def is_end_allowed(self) -> bool:
-        """Indicates whether this plan allows to set the end date."""
+    def is_start_and_end_allowed(self) -> bool:
+        """Indicates whether this plan allows to set the start and end dates."""
         return self.is_dev or bool(self.restatements)
 
     @property
@@ -160,33 +181,6 @@ class Plan:
     @property
     def missing_intervals(self) -> t.List[MissingIntervals]:
         """Returns a list of missing intervals."""
-        if self._missing_intervals is None:
-            previous_ids = [
-                SnapshotId(
-                    name=snapshot.name,
-                    identifier=snapshot.previous_version.fingerprint.to_identifier(),
-                )
-                for snapshot in self.snapshots
-                if snapshot.previous_version
-            ]
-
-            previous_snapshots = (
-                list(self._state_reader.get_snapshots(previous_ids).values())
-                if previous_ids
-                else []
-            )
-
-            end = self.end
-            self._missing_intervals = {
-                snapshot.version_get_or_generate(): missing
-                for snapshot, missing in self._state_reader.missing_intervals(
-                    previous_snapshots + list(self.snapshots),
-                    start=self.start,
-                    end=end,
-                    latest=end,
-                    restatements=self.restatements,
-                ).items()
-            }
         return [
             MissingIntervals(
                 snapshot_name=snapshot.name,
@@ -273,7 +267,7 @@ class Plan:
 
             # If any other snapshot specified breaking this child, then that child
             # needs to be backfilled as a part of the plan.
-            for upstream in self.added_and_directly_modified:
+            for upstream in self.directly_modified:
                 if child in upstream.indirect_versions:
                     data_version = upstream.indirect_versions[child][-1]
                     if data_version.is_new_version:
@@ -329,6 +323,38 @@ class Plan:
         # Return the most conservative categorization found in the snapshot's history
         return min(change_categories, key=lambda x: x.value)
 
+    @property
+    def _missing_intervals(self) -> t.Dict[str, Intervals]:
+        if self.__missing_intervals is None:
+            previous_ids = [
+                SnapshotId(
+                    name=snapshot.name,
+                    identifier=snapshot.previous_version.fingerprint.to_identifier(),
+                )
+                for snapshot in self.snapshots
+                if snapshot.previous_version
+            ]
+
+            previous_snapshots = (
+                list(self._state_reader.get_snapshots(previous_ids).values())
+                if previous_ids
+                else []
+            )
+
+            end = self.end
+            self.__missing_intervals = {
+                snapshot.version_get_or_generate(): missing
+                for snapshot, missing in self._state_reader.missing_intervals(
+                    previous_snapshots + list(self.snapshots),
+                    start=self._start or scheduler.earliest_start_date(self.snapshots),
+                    end=end,
+                    latest=end,
+                    restatements=self.restatements,
+                ).items()
+            }
+
+        return self.__missing_intervals
+
     def _add_restatements(self, restate_models: t.Iterable[str]) -> None:
         for table in restate_models:
             downstream = self._dag.downstream(table)
@@ -344,70 +370,86 @@ class Plan:
                 )
             self._restatements.update(downstream)
 
-    def _categorize_snapshots(self) -> t.Tuple[t.List[Snapshot], SnapshotMapping]:
-        """Automatically categorizes snapshots that can be automatically categorized and
-        returns a list of added and directly modified snapshots as well as the mapping of
-        indirectly modified snapshots.
+    def _build_directly_and_indirectly_modified(self) -> t.Tuple[t.List[Snapshot], SnapshotMapping]:
+        """Builds collections of directly and inderectly modified snapshots.
 
         Returns:
             The tuple in which the first element contains a list of added and directly modified
             snapshots while the second element contains a mapping of indirectly modified snapshots.
         """
-        queue = deque(self._dag.sorted())
-        added_and_directly_modified = []
+        directly_modified = []
         all_indirectly_modified = set()
 
-        while queue:
-            model_name = queue.popleft()
-
-            if model_name not in self.context_diff.snapshots:
-                continue
-
-            upstream_model_names = self._dag.upstream(model_name)
-
-            if not self.forward_only:
-                self._ensure_no_paused_forward_only_upstream(model_name, upstream_model_names)
-
-            snapshot = self.context_diff.snapshots[model_name]
-
+        for model_name, snapshot in self.context_diff.snapshots.items():
             if model_name in self.context_diff.modified_snapshots:
-                if self.forward_only and self.is_new_snapshot(snapshot):
-                    # In case of the forward only plan any modifications result in reuse of the
-                    # previous version.
-                    snapshot.set_version(snapshot.previous_version)
-
                 if self.context_diff.directly_modified(model_name):
-                    added_and_directly_modified.append(snapshot)
+                    directly_modified.append(snapshot)
                 else:
                     all_indirectly_modified.add(model_name)
-
-                    # set to breaking if an indirect child has no directly modified parents
-                    # that need a decision. this can happen when a revert to a parent causes
-                    # an indirectly modified snapshot to be created because of a new parent
-                    if not snapshot.version and not any(
-                        self.context_diff.directly_modified(upstream)
-                        and not self.context_diff.snapshots[upstream].version
-                        for upstream in upstream_model_names
-                    ):
-                        snapshot.set_version()
-
             elif model_name in self.context_diff.added:
-                if self.is_new_snapshot(snapshot):
-                    snapshot.set_version()
-                added_and_directly_modified.append(snapshot)
+                directly_modified.append(snapshot)
 
         indirectly_modified: SnapshotMapping = defaultdict(set)
-
-        for snapshot in added_and_directly_modified:
+        for snapshot in directly_modified:
             for downstream in self._dag.downstream(snapshot.name):
                 if downstream in all_indirectly_modified:
                     indirectly_modified[snapshot.name].add(downstream)
 
         return (
-            added_and_directly_modified,
+            directly_modified,
             indirectly_modified,
         )
 
+    def _categorize_snapshots(self) -> None:
+        """Automatically categorizes snapshots that can be automatically categorized and
+        returns a list of added and directly modified snapshots as well as the mapping of
+        indirectly modified snapshots.
+        """
+        for model_name, snapshot in self.context_diff.snapshots.items():
+            upstream_model_names = self._dag.upstream(model_name)
+
+            if not self.forward_only:
+                self._ensure_no_paused_forward_only_upstream(model_name, upstream_model_names)
+
+            if model_name in self.context_diff.modified_snapshots:
+                is_directly_modified = self.context_diff.directly_modified(model_name)
+
+                if self.is_new_snapshot(snapshot):
+                    if self.forward_only:
+                        # In case of the forward only plan any modifications result in reuse of the
+                        # previous version for non-seed models.
+                        # New snapshots of seed models are considered non-breaking ones.
+                        if not snapshot.is_seed_kind:
+                            snapshot.set_version(snapshot.previous_version)
+                            snapshot.change_category = SnapshotChangeCategory.FORWARD_ONLY
+                        else:
+                            snapshot.set_version()
+                            snapshot.change_category = SnapshotChangeCategory.NON_BREAKING
+                    elif self.auto_categorization_enabled and is_directly_modified:
+                        new, old = self.context_diff.modified_snapshots[model_name]
+                        change_category = categorize_change(
+                            new, old, config=self.categorizer_config
+                        )
+                        if change_category is not None:
+                            self.set_choice(new, change_category)
+
+                # set to breaking if an indirect child has no directly modified parents
+                # that need a decision. this can happen when a revert to a parent causes
+                # an indirectly modified snapshot to be created because of a new parent
+                if (
+                    not is_directly_modified
+                    and not snapshot.version
+                    and not any(
+                        self.context_diff.directly_modified(upstream)
+                        and not self.context_diff.snapshots[upstream].version
+                        for upstream in upstream_model_names
+                    )
+                ):
+                    snapshot.set_version()
+
+            elif model_name in self.context_diff.added and self.is_new_snapshot(snapshot):
+                snapshot.set_version()
+
     def _ensure_no_paused_forward_only_upstream(
         self, model_name: str, upstream_model_names: t.Iterable[str]
     ) -> None:
@@ -427,9 +469,13 @@ class Plan:
                     "3) recreate this plan in a forward-only mode."
                 )
 
-    def _ensure_valid_end(self, end: t.Optional[TimeLike]) -> None:
-        if end and not self.is_end_allowed:
-            raise PlanError("The end date can't be set for a production plan without restatements.")
+    def _ensure_valid_date_range(
+        self, start: t.Optional[TimeLike], end: t.Optional[TimeLike]
+    ) -> None:
+        if (start or end) and not self.is_start_and_end_allowed:
+            raise PlanError(
+                "The start and end dates can't be set for a production plan without restatements."
+            )
 
     def _ensure_no_forward_only_revert(self) -> None:
         """Ensures that a previously superseded breaking / non-breaking snapshot is not being
@@ -457,14 +503,6 @@ class Plan:
         if self.forward_only and self.context_diff.added:
             raise PlanError("New models can't be added as part of the forward-only plan.")
 
-    def _ensure_no_forward_only_seed_models(self) -> None:
-        if self.forward_only:
-            for snapshot in self.new_snapshots:
-                if snapshot.is_seed_kind:
-                    raise PlanError(
-                        f"Seed model '{snapshot.name}' can't be updated as part of the forward-only plan."
-                    )
-
 
 class PlanStatus(str, Enum):
     STARTED = "started"
