@@ -7,10 +7,19 @@ from enum import IntEnum
 
 from croniter import croniter_range
 from pydantic import validator
+from sqlglot import exp
 
 from sqlmesh.core import constants as c
 from sqlmesh.core.audit import Audit
-from sqlmesh.core.model import Model, SeedModel, kind, parse_model_name
+from sqlmesh.core.model import (
+    Model,
+    PythonModel,
+    SeedModel,
+    SqlModel,
+    kind,
+    parse_model_name,
+)
+from sqlmesh.core.model.meta import HookCall
 from sqlmesh.utils.date import (
     TimeLike,
     make_inclusive,
@@ -609,9 +618,9 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
     def is_incremental_by_unique_key_kind(self) -> bool:
         return self.model.kind.is_incremental_by_unique_key
 
-    @property
-    def is_snapshot_kind(self) -> bool:
-        return self.model.kind.is_snapshot
+    # @property
+    # def is_snapshot_kind(self) -> bool:
+    #     return self.model.kind.is_snapshot
 
     @property
     def is_embedded_kind(self) -> bool:
@@ -701,8 +710,23 @@ def fingerprint_from_model(
 
 
 def _model_data_hash(model: Model, physical_schema: str) -> str:
+    def serialize_hooks(hooks: t.List[HookCall]) -> t.Iterable[str]:
+        serialized = []
+        for hook in hooks:
+            if isinstance(hook, exp.Expression):
+                serialized.append(hook.sql())
+            else:
+                name, args = hook
+                serialized.append(
+                    f"{name}:"
+                    + ",".join(
+                        f"{k}={v.sql(identify=True, comments=False)}"
+                        for k, v in sorted(args.items())
+                    )
+                )
+        return serialized
+
     data = [
-        model.render_query().sql(identify=True, comments=False) if not model.is_seed else None,
         str(model.sorted_python_env),
         model.kind.name,
         model.cron,
@@ -710,14 +734,27 @@ def _model_data_hash(model: Model, physical_schema: str) -> str:
         physical_schema,
         *(model.partitioned_by or []),
         *(expression.sql(identify=True, comments=False) for expression in model.expressions or []),
+        *serialize_hooks(model.pre),
+        *serialize_hooks(model.post),
         model.stamp,
     ]
 
-    if isinstance(model.kind, kind.IncrementalByTimeRangeKind):
-        data.append(model.kind.time_column.column)
-        data.append(model.kind.time_column.format)
-    elif isinstance(model.kind, kind.IncrementalByUniqueKeyKind):
-        data.extend(model.kind.unique_key)
+    if isinstance(model, SqlModel):
+        data.append(model.query.sql(identify=True, comments=False))
+
+        for macro_name, macro in sorted(model.jinja_macros.root_macros.items(), key=lambda x: x[0]):
+            data.append(macro_name)
+            data.append(macro.definition)
+
+        for package in model.jinja_macros.packages.values():
+            for macro_name, macro in sorted(package.items(), key=lambda x: x[0]):
+                data.append(macro_name)
+                data.append(macro.definition)
+    elif isinstance(model, PythonModel):
+        data.append(model.entrypoint)
+        for column_name, column_type in model.columns_to_types.items():
+            data.append(column_name)
+            data.append(str(column_type))
     elif isinstance(model, SeedModel):
         data.append(str(model.kind.batch_size))
         data.append(model.seed.content)
@@ -725,6 +762,12 @@ def _model_data_hash(model: Model, physical_schema: str) -> str:
             data.append(column_name)
             data.append(column_type.sql())
 
+    if isinstance(model.kind, kind.IncrementalByTimeRangeKind):
+        data.append(model.kind.time_column.column)
+        data.append(model.kind.time_column.format)
+    elif isinstance(model.kind, kind.IncrementalByUniqueKeyKind):
+        data.extend(model.kind.unique_key)
+
     return _hash(data)
 
 
