@@ -60,7 +60,7 @@ def sushi_dbt_context(mocker: MockerFixture) -> Context:
 def init_and_plan_sushi_context(
     path: str, mocker: MockerFixture, start: TimeLike = "1 week ago"
 ) -> t.Tuple[Context, Plan]:
-    sushi_context = Context(path=path)
+    sushi_context = Context(path=path, config="test_config")
 
     for snapshot in sushi_context.snapshots.values():
         snapshot.set_version()
@@ -68,7 +68,8 @@ def init_and_plan_sushi_context(
     confirm = mocker.patch("sqlmesh.core.console.Confirm")
     confirm.ask.return_value = False
 
-    plan = sushi_context.plan("prod", start=start)
+    plan = sushi_context.plan("prod")
+    plan.set_start(start)
 
     return (sushi_context, plan)
 
