@@ -5,6 +5,7 @@ from pytest_mock.plugin import MockerFixture
 from sqlglot.expressions import DataType
 
 from sqlmesh.core import constants as c
+from sqlmesh.core.config import AutoCategorizationMode
 from sqlmesh.core.context import Context
 from sqlmesh.core.engine_adapter import EngineAdapter
 from sqlmesh.core.model import (
@@ -54,7 +55,6 @@ def test_model_removed(sushi_context: Context):
 
     sushi_context._models.pop("sushi.top_waiters")
     removed = ["sushi.top_waiters"]
-    [key for key in sushi_context.snapshots if key not in removed]
 
     def _validate_plan(context, plan):
         validate_plan_changes(plan, removed=removed)
@@ -202,7 +202,7 @@ def validate_query_change(
     ],
 )
 def test_model_kind_change(from_: ModelKindName, to: ModelKindName, sushi_context: Context):
-    environment = "prod"
+    environment = f"test_model_kind_change__{from_.value.lower()}__{to.value.lower()}"
     incremental_snapshot = sushi_context.snapshots["sushi.items"].copy()
 
     if from_ != ModelKindName.INCREMENTAL_BY_TIME_RANGE:
@@ -375,7 +375,9 @@ def test_no_override(sushi_context: Context) -> None:
         DataType.Type.INT,
         DataType.Type.BIGINT,
     )
-    plan = sushi_context.plan("prod", start=start(sushi_context))
+    plan = sushi_context.plan("prod")
+    plan.set_start(start(sushi_context))
+
     items = plan.context_diff.snapshots["sushi.items"]
     order_items = plan.context_diff.snapshots["sushi.order_items"]
     waiter_revenue = plan.context_diff.snapshots["sushi.waiter_revenue_by_day"]
@@ -443,7 +445,9 @@ def setup_rebase(
         DataType.Type.DOUBLE,
         DataType.Type.FLOAT,
     )
-    plan = context.plan("prod", start=start(context))
+    plan = context.plan("prod")
+    plan.set_start(start(context))
+
     plan_choice(plan, remote_choice)
     remote_versions = {snapshot.name: snapshot.version for snapshot in plan.snapshots}
     context.apply(plan)
@@ -599,10 +603,29 @@ def test_revert_after_downstream_change(sushi_context: Context):
     )
 
 
+@pytest.mark.integration
+@pytest.mark.core_integration
+def test_auto_categorization(sushi_context: Context):
+    environment = "dev"
+    sushi_context.config.auto_categorize_changes.sql = AutoCategorizationMode.FULL
+    initial_add(sushi_context, environment)
+
+    version = sushi_context.snapshots["sushi.waiter_as_customer_by_day"].version
+    fingerprint = sushi_context.snapshots["sushi.waiter_as_customer_by_day"].fingerprint
+
+    model = t.cast(SqlModel, sushi_context.models["sushi.waiters"])
+    sushi_context.upsert_model("sushi.waiters", query=model.query.select("'foo' AS foo"))  # type: ignore
+    apply_to_environment(sushi_context, environment)
+
+    assert sushi_context.snapshots["sushi.waiter_as_customer_by_day"].change_category is None
+    assert sushi_context.snapshots["sushi.waiter_as_customer_by_day"].fingerprint != fingerprint
+    assert sushi_context.snapshots["sushi.waiter_as_customer_by_day"].version == version
+
+
 def initial_add(context: Context, environment: str):
     assert not context.state_reader.get_environment(environment)
 
-    plan = context.plan(environment, start=start(context))
+    plan = context.plan(environment, start=start(context), create_from="nonexistent_env")
     validate_plan_changes(plan, added=context.models)
 
     context.apply(plan)
@@ -612,7 +635,7 @@ def initial_add(context: Context, environment: str):
 def apply_to_environment(
     context: Context,
     environment: str,
-    choice: SnapshotChangeCategory,
+    choice: t.Optional[SnapshotChangeCategory] = None,
     plan_validators: t.Optional[t.Iterable[t.Callable]] = None,
     apply_validators: t.Optional[t.Iterable[t.Callable]] = None,
 ):
@@ -621,10 +644,12 @@ def apply_to_environment(
 
     plan = context.plan(
         environment,
-        start=start(context),
         forward_only=choice == SnapshotChangeCategory.FORWARD_ONLY,
     )
-    plan_choice(plan, choice)
+    plan.set_start(start(context))
+
+    if choice:
+        plan_choice(plan, choice)
     for validator in plan_validators:
         validator(context, plan)
 
