@@ -6,8 +6,15 @@ from _pytest.monkeypatch import MonkeyPatch
 from pytest_mock.plugin import MockerFixture
 from sqlglot import exp, parse, parse_one
 
-from sqlmesh.core.macros import macro
-from sqlmesh.core.model import Model, SqlModel, load_model
+from sqlmesh.core.config import AutoCategorizationMode, CategorizerConfig
+from sqlmesh.core.model import (
+    IncrementalByTimeRangeKind,
+    Model,
+    SeedKind,
+    SqlModel,
+    create_seed_model,
+    load_model,
+)
 from sqlmesh.core.snapshot import (
     Snapshot,
     SnapshotChangeCategory,
@@ -16,12 +23,15 @@ from sqlmesh.core.snapshot import (
     fingerprint_from_model,
 )
 from sqlmesh.utils.date import to_datetime, to_timestamp
+from sqlmesh.utils.errors import SQLMeshError
+from sqlmesh.utils.jinja import JinjaMacroRegistry, MacroInfo
 
 
 @pytest.fixture
 def parent_model():
     return SqlModel(
         name="parent.tbl",
+        kind=IncrementalByTimeRangeKind(time_column="ds"),
         dialect="spark",
         query=parse_one("SELECT 1, ds"),
     )
@@ -31,6 +41,7 @@ def parent_model():
 def model():
     return SqlModel(
         name="name",
+        kind=IncrementalByTimeRangeKind(time_column="ds"),
         owner="owner",
         dialect="spark",
         cron="1 0 * * *",
@@ -83,6 +94,11 @@ def test_json(snapshot: Snapshot):
             "pre": [],
             "owner": "owner",
             "query": "SELECT @EACH(ARRAY(1, 2), x -> x), ds FROM parent.tbl",
+            "jinja_macros": {
+                "global_objs": {},
+                "packages": {},
+                "root_macros": {},
+            },
             "source_type": "sql",
         },
         "audits": [],
@@ -243,18 +259,16 @@ each_macro = lambda: "test"
 
 
 def test_fingerprint(model: Model, parent_model: Model):
-    macro.get_registry()
     fingerprint = fingerprint_from_model(model, models={})
 
     original_fingerprint = SnapshotFingerprint(
-        data_hash="2278368927",
+        data_hash="3042895307",
         metadata_hash="3589467163",
     )
 
     assert fingerprint == original_fingerprint
     assert fingerprint_from_model(model, physical_schema="x", models={}) != fingerprint
 
-    parent_fingerprint = fingerprint_from_model(parent_model, models={})
     with_parent_fingerprint = fingerprint_from_model(model, models={"parent.tbl": parent_model})
     assert with_parent_fingerprint != fingerprint
     assert int(with_parent_fingerprint.parent_data_hash) > 0
@@ -289,7 +303,7 @@ def test_fingerprint_seed_model():
     )
 
     expected_fingerprint = SnapshotFingerprint(
-        data_hash="941582290",
+        data_hash="2542007450",
         metadata_hash="2750000337",
     )
 
@@ -310,6 +324,38 @@ def test_fingerprint_seed_model():
     assert updated_actual_fingerprint.metadata_hash == expected_fingerprint.metadata_hash
 
 
+def test_fingerprint_jinja_macros(model: Model):
+
+    model = SqlModel(
+        **{
+            **model.dict(),
+            "jinja_macros": JinjaMacroRegistry(
+                root_macros={
+                    "test_macro": MacroInfo(
+                        definition="{% macro test_macro() %}a{% endmacro %}", depends_on=[]
+                    )
+                }
+            ),
+        }
+    )
+    fingerprint = fingerprint_from_model(model, models={})
+
+    original_fingerprint = SnapshotFingerprint(
+        data_hash="2665680291",
+        metadata_hash="3589467163",
+    )
+
+    fingerprint = fingerprint_from_model(model, models={})
+    assert fingerprint == original_fingerprint
+
+    model.jinja_macros.root_macros["test_macro"] = MacroInfo(
+        definition="{% macro test_macro() %}b{% endmacro %}", depends_on=[]
+    )
+    updated_fingerprint = fingerprint_from_model(model, models={})
+    assert updated_fingerprint.data_hash != original_fingerprint.data_hash
+    assert updated_fingerprint.metadata_hash == original_fingerprint.metadata_hash
+
+
 def test_stamp(model: Model):
     original_fingerprint = fingerprint_from_model(model, models={})
 
@@ -368,14 +414,17 @@ def test_table_name(snapshot: Snapshot):
     assert snapshot.table_name_for_mapping(is_dev=True) == "sqlmesh.name__3078928823"
 
 
-def test_categorize_change(make_snapshot):
+def test_categorize_change_sql(make_snapshot):
     old_snapshot = make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds")))
 
+    config = CategorizerConfig(sql=AutoCategorizationMode.SEMI)
+
     # A projection has been added.
     assert (
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, 2, ds"))),
             old=old_snapshot,
+            config=config,
         )
         == SnapshotChangeCategory.NON_BREAKING
     )
@@ -385,6 +434,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, fun(a * 2)::INT, ds"))),
             old=old_snapshot,
+            config=config,
         )
         == SnapshotChangeCategory.NON_BREAKING
     )
@@ -394,18 +444,21 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, 2, a, b, ds"))),
             old=old_snapshot,
+            config=config,
         )
         == SnapshotChangeCategory.NON_BREAKING
     )
 
     # No change.
-    assert categorize_change(old_snapshot, old_snapshot) is None
+    with pytest.raises(SQLMeshError):
+        categorize_change(old_snapshot, old_snapshot, config=config)
 
     # A projection has been removed.
     assert (
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select ds"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -415,6 +468,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 2, ds"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -424,6 +478,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select ds, 1"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -433,6 +488,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds WHERE a = 2"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -442,6 +498,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds FROM test_table"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -451,6 +508,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select DISTINCT 1, ds"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -460,6 +518,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds, explode(a)"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -471,6 +530,7 @@ def test_categorize_change(make_snapshot):
                 SqlModel(name="a", query=parse_one("select 1, ds, explode_outer(a)"))
             ),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -480,6 +540,7 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds, posexplode(a)"))),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -491,6 +552,7 @@ def test_categorize_change(make_snapshot):
                 SqlModel(name="a", query=parse_one("select 1, ds, posexplode_outer(a)"))
             ),
             old=old_snapshot,
+            config=config,
         )
         is None
     )
@@ -500,6 +562,178 @@ def test_categorize_change(make_snapshot):
         categorize_change(
             new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds, unnest(a)"))),
             old=old_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # A metadata change occurred
+    assert (
+        categorize_change(
+            new=make_snapshot(SqlModel(name="a", query=parse_one("select 1, ds"), owner="foo")),
+            old=old_snapshot,
+            config=config,
+        )
+        is SnapshotChangeCategory.NON_BREAKING
+    )
+
+
+def test_categorize_change_seed(make_snapshot, tmp_path):
+    config = CategorizerConfig(seed=AutoCategorizationMode.SEMI)
+    model_name = "test_db.test_seed_model"
+    seed_path = tmp_path / "seed.csv"
+    model_kind = SeedKind(path=str(seed_path.absolute()))
+
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_c
+1,text_a,1.0
+2,text_b,2.0"""
+        )
+
+    original_snapshot = make_snapshot(create_seed_model(model_name, model_kind))
+
+    # New column.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_d,col_b,col_c
+1,,text_a,1.0
+2,test,text_b,2.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        == SnapshotChangeCategory.NON_BREAKING
+    )
+
+    # Column removed.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_c
+1,1.0
+2,2.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # Column renamed.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_d
+1,text_a,1.0
+2,text_b,2.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # New row.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_c
+1,text_a,1.0
+2,text_b,2.0
+3,text_c,3.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # Deleted row.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_c
+1,text_a,1.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # Numeric column changed.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_c
+1,text_a,1.0
+2,text_b,3.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # Text column changed.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_c
+1,text_a,1.0
+2,text_c,2.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
+        )
+        is None
+    )
+
+    # Column type changed.
+    with open(seed_path, "w") as fd:
+        fd.write(
+            """
+col_a,col_b,col_c
+1,text_a,1.0
+2.0,text_b,2.0"""
+        )
+
+    assert (
+        categorize_change(
+            new=make_snapshot(create_seed_model(model_name, model_kind)),
+            old=original_snapshot,
+            config=config,
         )
         is None
     )
