@@ -1,4 +1,4 @@
-import Split, { SplitProps } from 'react-split'
+import Split, { type SplitProps } from 'react-split'
 import './SplitPane.css'
 
 export default function SplitPane({
@@ -6,17 +6,22 @@ export default function SplitPane({
   children,
   sizes,
   minSize,
+  maxSize,
   direction,
+  expandToMin,
+  snapOffset,
 }: SplitProps): JSX.Element {
   return (
     <Split
       className={className}
       sizes={sizes}
-      expandToMin={false}
+      expandToMin={expandToMin}
       gutterAlign="center"
       gutterSize={3}
       direction={direction}
       minSize={minSize}
+      maxSize={maxSize}
+      snapOffset={snapOffset}
     >
       {children}
     </Split>
