@@ -1,8 +1,8 @@
 import type { File } from '../api/client'
-import { ModelDirectory } from './directory'
-import { InitialArtifact, ModelArtifact } from './artifact'
+import { type ModelDirectory } from './directory'
+import { type InitialArtifact, ModelArtifact } from './artifact'
 
-interface InitialFile extends InitialArtifact, File {
+export interface InitialFile extends InitialArtifact, File {
   content: string
   extension: string
   is_supported: boolean
@@ -42,4 +42,15 @@ export class ModelFile extends ModelArtifact<InitialFile> {
   get isChanged(): boolean {
     return this.content !== this.initial.content
   }
+
+  get isSQLMeshModel(): boolean {
+    return (
+      ['.sql', '.py'].includes(this.extension) &&
+      this.path.startsWith('models/')
+    )
+  }
+
+  get isSQLMeshSeed(): boolean {
+    return ['.csv'].includes(this.extension) && this.path.startsWith('seeds/')
+  }
 }
